import fs from "fs"
import path_lib from "path"
import { Logrotate } from "syrus4-nodejs"

export class File {
	constructor( config ) { //{ host, path }
		this.onConnect = null;
		this.onDisconnect = null;
		this.onMessage = null;
		this.path = config.host || config.path;

		if (this.path.includes('$MEDIA_PATH')) {
			this.path = this.path.replace('$MEDIA_PATH', '/media/mmcblk0p1');
		} else if ( this.path.includes('$USER_PATH')) {
			this.path = this.path.replace('$USER_PATH', '/data/users/syrus4g');
		}

		try {
			// create directory if it doesn't exist
			if (!fs.existsSync(this.path)){
				fs.mkdirSync(path_lib.dirname(this.path), {recursive: true})
				// touch the file
				try {
					time = new Date()
					fs.utimesSync(this.path, time, time);
				} catch (err) {
					fs.closeSync(fs.openSync(this.path, 'w'));
				}
			}

			// Configure the file rotation
			//const name = config.host.split('/').pop();
			Logrotate.setConfiguration(config.name, config.host, config.rotate, config.size, config.compress);

		} catch (error) {
			console.error("Unable to create path ", this.path);
			throw error;
		}
	}

	get disable_welcome_message() { return true }
	get no_disconnects() { return true }
	get no_acks() { return true } 

	destroy(){
		return;
	}

	connect() {
		if (this.onConnect) this.onConnect();
	}

	disconnect() {
		if (this.onDisconnect) this.onDisconnect();
	}

	send(data, next) {
		if (!next) { next = () => { } }
		try {
			// Check if is a string but not taip
			if (typeof data === 'string' && data[0] != '>') {
				fs.appendFileSync(this.path, `${data}` + "\n", {
					encoding: "utf8"
				});
			} else {
				fs.appendFileSync(this.path, JSON.stringify(data) + "\n", {
					encoding: "utf8"
				});
			}

			next()
		} catch (error) {
			console.error(error);
			next(error)
		}
	}
}
