
export class CSV {
    constructor({ name, client, onCommand, autoDequeue = false }) {
        this.name = name;
        if (onCommand) this.onCommand = onCommand;
        this.autoDequeue = autoDequeue;
        this.client = client;
        this.client.connect();
        this.client.onCommand = this.onCommand;
        this.variablesCount = 0;
        this.firstLoad = true;
    }


    // Must be exposed by all the protocols that supports send to destinations
	/**
	 * default send command this is called from the main, process the event To the protocol
	 * @param {Object<any>} variables syrusjs_variables copy
	 * @param {Object<any>} opts event defintion parameters, min_duration, trigger rater,etc
	 * @param {Object<any>} fields fieldset from the event
	 * @param {string} label the label from the event
	 * @param {string} code the code from the event
	 */
    send( variables, opts, fields, label, code ) {
        //console.log('CSV Send:', variables, opts, fields, label, code);   

        // Print first header - Verify the number of variables hasn't grow
        if (this.firstLoad) {
            // Get fieldset order
            let header = `timestamp,date_time,event_label,event_code,`
            for (const field in fields) {
        
                if ( !field.includes('$') ) {
                    // Field defined by alias like VO: $counters.globals.odometer
                    for (const element in variables) {
                        if (element == fields[field]) {
                            header += `${field},`;
                            this.variablesCount += 1;
                        }
                    }

                } else {
                    // Field defined by fieldset like $io, $gnss, $net ..
                    const fieldArray = [];
                    for (const element in variables) {
                        if (element == field) {
                            header += `${element},`;
                            this.variablesCount += 1;
                        }
                    }
                }

            }

            // Removes last coma
            header = header.slice(0,-1);
            

            this.client.send(`${header}`, (error) => {
                if (error) console.error(error);
            });

            console.log('CSV Header added:', this.variablesCount, header);
            this.firstLoad = false;
        }

        let payload = ''
        let count = 0;
        const replaceRegex = /,/g
        // Print values in CSV format
        for (const field in fields) {
            if ( !field.includes('$') ) {
                // Field defined by alias like VO: $counters.globals.odometer
                for (const element in variables) {
                    if (element == fields[field]) {
                        if (`${variables[element]}`.includes(',')) {
                            const modified = `${variables[element]}`.replace(replaceRegex,'  ');
                            payload += `${modified},`;
                            count += 1;
                        } else {
                            payload += `${variables[element]},`; 
                            count += 1; 
                        }
                    }
                }

            } else {
                // Field defined by fieldset like $io, $gnss, $net ..
                const fieldArray = [];
                for (const element in variables) {
                    if (element == field) {
                        if (`${variables[element]}`.includes(',')) {
                            const modified = `${variables[element]}`.replace(replaceRegex,'  ');
                            payload += `${modified},`;
                            count += 1;
                        } else {
                            payload += `${variables[element]},`;
                            count += 1;
                        }
                    }
                }
            }

        }   

        // Removes last coma
        payload = payload.slice(0,-1);

        // Failsafe for variables just created
        if (this.variablesCount != count) {
            this.firstLoad = true;
        }

        //this.push('true,true,true');
        this.client.send(`${new Date().getTime()},${new Date().toISOString()},${label},${code},${payload}`, (error) => {
            if (error) console.error(error);
        });

    } 

}