import { Counters as S4Counters } from "syrus4-nodejs"
import { syrusjs_variables, structure, counters } from "./globals.js"

let intervalId = 0;

/**
 * create a new S4 Counters and stored in global counters
 * @param {*} config counters parameters
 */
export async function createCounters(config) {
	
	if ( config.signal != undefined ) {

		const name = config.name.toLowerCase();
		
		// Initit custom counter params count,time,distance
		const initial_distance =  ( isNaN(syrusjs_variables[`$counters.${name}.total_distance`]) ) ? 0 : syrusjs_variables[`$counters.${config.name.toLowerCase()}.total_distance`];
		syrusjs_variables[`$counters.${name}.total_distance`] = Number(initial_distance);
		syrusjs_variables[`$counters.${name}.distance_ref`] = 0;
		syrusjs_variables[`$counters.${name}.distance_increment`] = 0;
		syrusjs_variables[`$counters.${name}.distance_increment_ref`] = 0;

		const initial_time =  ( isNaN(syrusjs_variables[`$counters.${name}.total_time`]) ) ? 0 : syrusjs_variables[`$counters.${name}.total_time`];
		syrusjs_variables[`$counters.${name}.total_time`] = Number(initial_time);
		syrusjs_variables[`$counters.${name}.time_ref`] = 0;
		syrusjs_variables[`$counters.${name}.time_increment`] = 0;

		const initial_count  = ( isNaN(syrusjs_variables[`$counters.${name}.total_count`]) ) ?  0 :  syrusjs_variables[`$counters.${name}.total_count`];
		syrusjs_variables[`$counters.${name}.total_count`] = Number(initial_count);

		syrusjs_variables[`$counters.${name}.state`] = false;


		const start = Boolean(config.start) || false;
		const total_time = Number(syrusjs_variables[`$counters.${name}.total_time`]) | 0;
		const total_distance = Number(syrusjs_variables[`$counters.${name}.total_distance`]) | 0;

		counters[`${name}`] = {
			type: 'incremental',
			signal: config.signal,
			running: start,

			currentTime: total_time, 
			previousTime: total_time,

			time_increment: 0,
			total_time: 0,
			
			currentDistance: total_distance,
			previousDistance: total_distance,

			distance_increment: 0,
			total_distance: 0,

			total_count: 0
		};



		// Add real counter and watcher used for distance
		if ( counters[`${name}`].watcher == undefined ) {

			//console.log('CreatingCounter ...', name, counters);
			S4Counters.startCounters({ name: name , forceSet: true, distance: 0 }).then( () => {


				//console.log('Counter started with parameters');
				counters[name].watcher = S4Counters.watchCounters( name, values => {
						onCountersChange(values, name);
					},
					console.log('error startingCounter')
				);

				// Check if second interval already started
				if (intervalId == 0) {
					// Start counters interval (one for all)
					intervalId = setInterval( () => countersCallback(), 1000);
				}  

				//console.log('createCounters:', counters);
			});
		} 


	} else {
		// Only core counters
		await S4Counters.startCounters(config);
		counters[config.name] = S4Counters.watchCounters(
			config.name,
			values => {
				onCountersChange(values, config.name);
			},
			console.error
		);

		console.log('createCounters:', counters);
	}
}

/**
 * start/play the counters
 * @param {*} name name of the counters
 */
export function startCounters(name) {

	if (counters[`${name}`].signal != undefined) {

		console.log('startCounter:', name )

		// Add running state for that counter
		counters[`${name}`].running = true;

		// Check if second interval already started
		if (intervalId == 0) {
			// Start counters interval (one for all)
			intervalId = setInterval( () => countersCallback(), 1000);
		}  

		if ( counters[`${name}`].watcher != undefined ) {
			S4Counters.startCounters({ name: name })
		}	
		
	} else {
		S4Counters.startCounters({ name: name });	
	}
}

/**
 * stop/pause the counters
 * @param {*} name name of the counters
 */
export function stopCounters(name) {
	if (counters[`${name}`].signal != undefined) {
		console.log('stopCounter:', name )

		// Enable state one shot again to avoid always true state
		syrusjs_variables[`$counters.${name}.state`] = false;

		S4Counters.stopCounters(name);

		counters[`${name}`].running = false;
	} else {
		S4Counters.stopCounters(name);
	}
}

/**
 * callback to execute when a counter is updated, its update the syrusjs variables
 * @param {*} counter the new data
 * @param {*} name name of the counters
 */
function onCountersChange(counter, name) {
	//console.log('onCounterChange:', counter, name);
	for (const key in counter) {
		syrusjs_variables[`$counters.${name}.${key.toLowerCase()}`] = counter[key];
	}
}

/**
 * Totally delete the counters from the name passed as argument
 * @param {*} name name of the counters
 */
export function deleteCounters(name) {		
	if (counters[`${name}`].signal != undefined) {
		if (counters[name]) counters[name].watcher.unsubscribe();
		S4Counters.deleteCounters(name)
		delete counters[name];
		console.log('deleteCounters:', name);
	} else {
		if (counters[name]) counters[name].unsubscribe();
		S4Counters.deleteCounters(name)
		delete counters[name];
	}

}

/**
 * Restart the counters from the name passed as argument
 * @param {*} name name of the counters
 */
export function resetCounters(name) {
	console.log('resetCounters:', name, counters[`${name}`].signal);
	if (counters[`${name}`].signal != undefined) {
		// Set everything to zero
		syrusjs_variables[`$counters.${name}.total_distance`] = 0;
		syrusjs_variables[`$counters.${name}.distance_ref`] = 0;
		syrusjs_variables[`$counters.${name}.distance_increment`] = 0;
		syrusjs_variables[`$counters.${name}.distance_increment_ref`] = 0;
		
		syrusjs_variables[`$counters.${name}.total_time`] = 0;
		syrusjs_variables[`$counters.${name}.time_ref`] = 0;
		syrusjs_variables[`$counters.${name}.time_increment`] = 0;

		syrusjs_variables[`$counters.${name}.total_count`] = 0;


		// Reset core counters too
		S4Counters.resetCounters(name);

		counters[`${name}`].running = false;
	} else {
		S4Counters.resetCounters(name);
	}
}

function countersCallback( ) {

	// For every counter created get the signal state and update the values
	for (const name in counters) {
		
		const counter = counters[name]
		if ( counter.signal != undefined  && counter.running == true) {

			const signal = structure.signals[counter.signal];

			// Main counting function
			if ( signal.state == true ) { 		

				// Increase time count
				const time = Number(syrusjs_variables[`$counters.${name}.total_time`]) + Number(1);
				syrusjs_variables[`$counters.${name}.total_time`] = time;

				// Increase total number count
				if ( !syrusjs_variables[`$counters.${name}.state`] ) {

					// Add one more count
					const count  = Number(syrusjs_variables[`$counters.${name}.total_count`]) + Number(1);
					syrusjs_variables[`$counters.${name}.total_count`] = count;

					// Disable state one shot
					syrusjs_variables[`$counters.${name}.state`] = true;

					// Start counting new distance interval
					syrusjs_variables[`$counters.${name}.distance_ref`] = Number(syrusjs_variables[`$counters.${name}.distance`]);

				} else {

					// Increment counting distance interval
					if ( Number(syrusjs_variables[`$counters.${name}.distance`]) != Number(syrusjs_variables[`$counters.${name}.distance_ref`]) ) {
						// Distance has changed! get increment distance
						const added_distance =  Number(syrusjs_variables[`$counters.${name}.distance`]) - Number(syrusjs_variables[`$counters.${name}.distance_ref`]);
						syrusjs_variables[`$counters.${name}.total_distance`]  =  Number(syrusjs_variables[`$counters.${name}.total_distance`]) + Number(added_distance);

						// Update last distance reference	
						syrusjs_variables[`$counters.${name}.distance_ref`] = Number(syrusjs_variables[`$counters.${name}.distance`]);
					}
				}

			} else {

				if (syrusjs_variables[`$counters.${name}.state`] == true ) {
						console.log(` -> Count stopped[${name}]:`, 
						syrusjs_variables[`$counters.${name}.total_count`], 
						syrusjs_variables[`$counters.${name}.total_time`], 
						syrusjs_variables[`$counters.${name}.time_increment`],  
						syrusjs_variables[`$counters.${name}.total_distance`], 
						syrusjs_variables[`$counters.${name}.distance_increment`]);
				}

				// Enable state one shot again
				syrusjs_variables[`$counters.${name}.state`] = false;
			}		

			/** 	
			console.log('countersCallbacks:',name, 
				syrusjs_variables[`$counters.${name}.state`], 
				syrusjs_variables[`$counters.${name}.total_count`], 
				syrusjs_variables[`$counters.${name}.total_time`], 
				syrusjs_variables[`$counters.${name}.time_increment`],  
				syrusjs_variables[`$counters.${name}.total_distance`], 
				syrusjs_variables[`$counters.${name}.distance_increment`]);*/
		}

	}
}

export function updateIncrements() {
	// If this function entered means a new event is about to go out

	for (const name in counters) {
		const counter = counters[name]

		if (counter.signal != undefined && counter.running == true) {
	
			// TIME -- Get current event timestamp	and  Update time increment from last sended event
			counters[name].currentTime =  syrusjs_variables[`$counters.${name}.total_time`]; 
			counters[name].time_increment = counters[name].currentTime - counters[name].previousTime;

			// DISTANCE -- Get current event distance and update disntance increment from last sended event
			counters[name].currentDistance = syrusjs_variables[`$counters.${name}.total_distance`];
			counters[name].distance_increment = counters[name].currentDistance - counters[name].previousDistance;

			// Update previous with current
			counters[name].previousTime = counter.currentTime  | syrusjs_variables[`$counters.${name}.total_time`];
			counters[name].previousDistance = counter.currentDistance | syrusjs_variables[`$counters.${name}.total_distance`];

			// Save Variables
			syrusjs_variables[`$counters.${name}.time_increment`] = counters[name].time_increment;
			syrusjs_variables[`$counters.${name}.distance_increment`] = counters[name].distance_increment;
		}
	}
}